// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator 0.16.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.Cdn
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading;
    using System.Threading.Tasks;
    using Microsoft.Rest;
    using Microsoft.Rest.Azure;
    using Models;

    /// <summary>
    /// ProfilesOperations operations.
    /// </summary>
    public partial interface IProfilesOperations
    {
        /// <summary>
        /// Lists the CDN profiles within an Azure subscitption.
        /// </summary>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<IEnumerable<Profile>>> ListBySubscriptionIdWithHttpMessagesAsync(Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Lists the CDN profiles within a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<IEnumerable<Profile>>> ListByResourceGroupWithHttpMessagesAsync(string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Gets a CDN profile with the specified parameters.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Profile>> GetWithHttpMessagesAsync(string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Creates a new CDN profile with the specified parameters.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='profileProperties'>
        /// Profile properties needed for creation.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Profile>> CreateWithHttpMessagesAsync(string profileName, ProfileCreateParameters profileProperties, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Creates a new CDN profile with the specified parameters.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='profileProperties'>
        /// Profile properties needed for creation.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Profile>> BeginCreateWithHttpMessagesAsync(string profileName, ProfileCreateParameters profileProperties, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Updates an existing CDN profile with the specified parameters.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='tags'>
        /// Profile tags
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Profile>> UpdateWithHttpMessagesAsync(string profileName, string resourceGroupName, IDictionary<string, string> tags, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Updates an existing CDN profile with the specified parameters.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='tags'>
        /// Profile tags
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Profile>> BeginUpdateWithHttpMessagesAsync(string profileName, string resourceGroupName, IDictionary<string, string> tags, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Deletes an existing CDN profile with the specified parameters.
        /// Deleting a profile will result in the deletion of all
        /// subresources including endpoints, origins and custom domains.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse> DeleteIfExistsWithHttpMessagesAsync(string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Deletes an existing CDN profile with the specified parameters.
        /// Deleting a profile will result in the deletion of all
        /// subresources including endpoints, origins and custom domains.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse> BeginDeleteIfExistsWithHttpMessagesAsync(string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Generates a dynamic SSO URI used to sign in to the CDN
        /// Supplemental Portal used for advanced management tasks, such as
        /// Country Filtering, Advanced HTTP Reports, and Real-time Stats and
        /// Alerts. The SSO URI changes approximately every 10 minutes.
        /// </summary>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<SsoUri>> GenerateSsoUriWithHttpMessagesAsync(string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
    }
}
